@@ -65,6 +65,8 @@ FSBackupset:
 
 from __future__ import unicode_literals
 
+from past.builtins import basestring
+
 from ..backupset import Backupset
 from ..client import Client
 from ..exception import SDKException
@@ -84,7 +86,8 @@ class FSBackupset(Backupset):
             from_time=None,
             to_time=None,
             fs_options=None,
-            restore_jobs=None):
+            restore_jobs=None
+    ):
         """Restores the files/folders specified in the input paths list to the same location.
 
             Args:
@@ -110,14 +113,21 @@ class FSBackupset(Backupset):
                     default: None
 
                 fs_options      (dict)          -- dictionary that includes all advanced options
+
                     options:
+
                         all_versions        : if set to True restores all the versions of the
                                                 specified file
+
                         versions            : list of version numbers to be backed up
+
                         validate_only       : To validate data backed up for restore
 
+                        no_of_streams   (int)       -- Number of streams to be used for restore
+
                 restore_jobs    (list)          --  list of jobs to be restored if the job is index free restore
 
+
             Returns:
                 object - instance of the Job class for this restore job
 
@@ -133,6 +143,11 @@ class FSBackupset(Backupset):
         """
         self._instance_object._restore_association = self._backupset_association
 
+        if fs_options is not None and fs_options.get('no_of_streams', 1) > 1 and not fs_options.get('destination_appTypeId', False):
+            fs_options['destination_appTypeId'] = int(self._client_object.agents.all_agents.get('file system', self._client_object.agents.all_agents.get('windows file system', self._client_object.agents.all_agents.get('linux file system', self._client_object.agents.all_agents.get('big data apps', self._client_object.agents.all_agents.get('cloud apps'))))))
+            if not fs_options['destination_appTypeId']:
+                del fs_options['destination_appTypeId']
+
         return self._instance_object._restore_in_place(
             paths=paths,
             overwrite=overwrite,
@@ -155,7 +170,8 @@ class FSBackupset(Backupset):
             from_time=None,
             to_time=None,
             fs_options=None,
-            restore_jobs=None):
+            restore_jobs=None
+    ):
         """Restores the files/folders specified in the input paths list to the input client,
             at the specified destionation location.
 
@@ -188,19 +204,30 @@ class FSBackupset(Backupset):
                     default: None
 
                 fs_options      (dict)             -- dictionary that includes all advanced options
+
                     options:
+
                         preserve_level      : preserve level option to set in restore
+
                         proxy_client        : proxy that needed to be used for restore
+
                         impersonate_user    : Impersonate user options for restore
+
                         impersonate_password: Impersonate password option for restore
                                                 in base64 encoded form
+
                         all_versions        : if set to True restores all the versions of the
                                                 specified file
+
                         versions            : list of version numbers to be backed up
+
                         validate_only       : To validate data backed up for restore
 
+                        no_of_streams   (int)       -- Number of streams to be used for restore
+
                 restore_jobs    (list)          --  list of jobs to be restored if the job is index free restore
 
+
             Returns:
                 object - instance of the Job class for this restore job
 
@@ -220,6 +247,17 @@ class FSBackupset(Backupset):
         """
         self._instance_object._restore_association = self._backupset_association
 
+        if not isinstance(client, (basestring, Client)):
+            raise SDKException('Subclient', '101')
+
+        if isinstance(client, basestring):
+            client = Client(self._commcell_object, client)
+
+        if fs_options is not None and fs_options.get('no_of_streams', 1) > 1 and not fs_options.get('destination_appTypeId', False):
+            fs_options['destination_appTypeId'] = int(client.agents.all_agents.get('file system', client.agents.all_agents.get('windows file system', client.agents.all_agents.get('linux file system', client.agents.all_agents.get('big data apps', client.agents.all_agents.get('cloud apps'))))))
+            if not fs_options['destination_appTypeId']:
+                del fs_options['destination_appTypeId']
+
         return self._instance_object._restore_out_of_place(
             client=client,
             destination_path=destination_path,
@@ -542,14 +580,14 @@ class FSBackupset(Backupset):
         if restore_options.get('VcenterServerName'):
 
             vm_option['vmInfo']['proxyClient'][
-            'clientName'] = restore_options.get('VirtualizationClient', None)
+                'clientName'] = restore_options.get('VirtualizationClient', None)
 
             response_json['taskInfo']['subTasks'][0]['options'][
-            'restoreOptions']['virtualServerRstOption']['diskLevelVMRestoreOption'][
-                'esxServerName'] = restore_options.get('VcenterServerName', None)
+                'restoreOptions']['virtualServerRstOption']['diskLevelVMRestoreOption'][
+                    'esxServerName'] = restore_options.get('VcenterServerName', None)
 
             response_json['taskInfo']['subTasks'][0]['options']['restoreOptions']['destination'][
-            'destClient']['clientName'] = restore_options.get('VirtualizationClient', None)
+                'destClient']['clientName'] = restore_options.get('VirtualizationClient', None)
 
             vm_option['vmInfo']['vmLocation']['hostName'] = restore_options.get(
                 'EsxServer')
@@ -557,14 +595,14 @@ class FSBackupset(Backupset):
             vm_option['vmInfo']['vmLocation']['vCenter'] = restore_options.get('VcenterServerName')
 
         if restore_options.get('HyperVInstance'):
-            if restore_options.get('OsType')=='UNIX':
+            if restore_options.get('OsType') == 'UNIX':
                 vm_option['vendor'] = 'MICROSOFT'
             response_json['taskInfo']['subTasks'][0]['options'][
-            'restoreOptions']['virtualServerRstOption']['diskLevelVMRestoreOption'][
-                'esxServerName'] = restore_options.get('HyperVInstance', None)
+                'restoreOptions']['virtualServerRstOption']['diskLevelVMRestoreOption'][
+                    'esxServerName'] = restore_options.get('HyperVInstance', None)
 
             response_json['taskInfo']['subTasks'][0]['options']['restoreOptions']['destination'][
-            'destClient']['clientName'] = restore_options.get('HyperVInstance', None)
+                'destClient']['clientName'] = restore_options.get('HyperVInstance', None)
 
             vm_option['vmInfo']['vmLocation']['hostName'] = restore_options.get(
                 'HyperVHost')
@@ -590,18 +628,18 @@ class FSBackupset(Backupset):
                 'clientName'] = restore_options.get('CloneClientName', None)
             vm_option['oneTouchResponse']['clients'][0]['newclient'][
                 'hostName'] = restore_options.get('CloneClientName', None)
-        if restore_options.get('OsType')=='UNIX':
+        if restore_options.get('OsType') == 'UNIX':
             vm_option['oneTouchResponse']['clients'][0]['newclient'][
-            'hostName'] = ''
+                'hostName'] = ''
             vm_option['oneTouchResponse']['clients'][0]['backupSet'][
-            'backupsetName'] = self._properties['backupSetEntity']['backupsetName']
+                'backupsetName'] = self._properties['backupSetEntity']['backupsetName']
             if (response_json['taskInfo']['subTasks'][0]['options']['adminOpts'][
-                'vmProvisioningOption']['virtualMachineOption'][0]['oneTouchResponse']
-                ['hwconfig']['mem_size']) < 4096:
+                    'vmProvisioningOption']['virtualMachineOption'][0]['oneTouchResponse'][
+                        'hwconfig']['mem_size']) < 4096:
                 vm_option['oneTouchResponse']['hwconfig']['mem_size'] = 4096
         if restore_options.get('UseDhcp'):
             vm_option['oneTouchResponse']['clients'][0]['netconfig']['ipinfo']['interfaces'][0][
-            'protocols'][0]['useDhcp'] = True
+                'protocols'][0]['useDhcp'] = True
 
         response_json['taskInfo']['subTasks'][0]['options']['restoreOptions']['destination'][
             'inPlace'] = False
@@ -1134,11 +1172,9 @@ class FSBackupset(Backupset):
 
         flag, response = self._cvpysdk_object.make_request('DELETE', self._services['DELETE_BLR_PAIR']%blrid)
 
-        if response.status_code != 200 and flag ==  False:
+        if response.status_code != 200 and flag == False:
             raise SDKException('Response', '101', self._update_response_(response.text))
 
-
-
     def create_fsblr_replication_pair(self, srcclientid, destclientid, rpstoreid=None, replicationtype=None,
                                       **replication_options):
         """"Create granular replication pair  json
@@ -1327,4 +1363,3 @@ class FSBackupset(Backupset):
                 raise SDKException('Response', '102')
         else:
             raise SDKException('Response', '101', self._update_response_(response.text))
-
