@@ -71,6 +71,8 @@ Plan
 
     _update_plan_props()        -- method to update plan properties
 
+    _get_associated_entities()  -- method to get list of entities associated to a plan
+
     derive_and_add()            -- add new plan by deriving from the parent Plan object
 
     plan_name                   --  returns the name of the plan
@@ -100,6 +102,8 @@ Plan Attributes
     **schedule_policies**       --  returns the schedule policy of the plan
 
     **subclient_policy**        --  returns the subclient policy of the plan
+
+    **associated_entities**     --  returns all the backup entties associted with the plan
 """
 
 from __future__ import unicode_literals
@@ -643,6 +647,7 @@ class Plan(object):
         self._override_entities = None
         self._parent_plan_id = None
         self._addons = []
+        self._associated_entities = {}
         self.refresh()
 
     def __repr__(self):
@@ -742,6 +747,8 @@ class Plan(object):
                 if 'parent' in self._plan_properties['summary']:
                     self._parent_plan_id = self._plan_properties['summary']['parent']['planId']
 
+                self._get_associated_entities()
+
                 return self._plan_properties
             else:
                 raise SDKException('Response', '102')
@@ -1041,6 +1048,31 @@ class Plan(object):
             response_string = self._commcell_object._update_response_(response.text)
             raise SDKException('Response', '101', response_string)
 
+    def _get_associated_entities(self):
+        """Gets all the backup entities associated with the plan.
+
+            Returns:
+                dict - dictionary containing list of entities that are
+                       associated with the plan.
+
+            Raises:
+                SDKException:
+                    if response is empty
+
+                    if response is not success
+        """
+        request_url = self._services['ASSOCIATED_ENTITIES'] % (self._plan_id)
+        flag, response = self._cvpysdk_object.make_request(
+            'GET', request_url
+        )
+
+        if flag:
+            if response.json() and 'entities' in response.json():
+                self._associated_entities = response.json()['entities']
+        else:
+            response_string = self._commcell_object._update_response_(response.text)
+            raise SDKException('Response', '101', response_string)
+
     @property
     def plan_id(self):
         """Treats the plan id as a read-only attribute."""
@@ -1162,6 +1194,11 @@ class Plan(object):
         """Treats the plan subclient policy as a read-only attribute"""
         return self._child_policies['subclientPolicyIds']
 
+    @property
+    def associated_entities(self):
+        """getter for the backup entities associated with the plan"""
+        return self._associated_entities
+
     def refresh(self):
         """Refresh the properties of the Plan."""
         self._properties = self._get_plan_properties()
