@@ -44,6 +44,8 @@ MediaAgents:
 
     get(media_agent_name)       --  returns the instance of MediaAgent class
     of the media agent specified
+    
+    delete(media_agent)     --  Deletes the media agent from the commcell.
 
     refresh()                   --  refresh the media agents associated with the commcell
 
@@ -64,6 +66,16 @@ MediaAgent:
 
     _initialize_media_agent_properties()    --  initializes media agent properties
 
+    enable_power_management()               --  Enable VM Management (power management)
+
+    _perform_power_operation()              --  Performs power operation (power-on/power-off)
+
+    power_on()                              --  Power-on MediaAgent if VM management is enabled
+
+    power_off()                             --  Power-off MediaAgent if VM management is enabled
+
+    wait_for_power_status()                 -- Waits till the expected power status is not achieved
+
     media_agent_name()                      --  returns media agent name
 
     media_agent_id()                        --  returns media agent id
@@ -113,19 +125,28 @@ DiskLibrary:
 
     _get_library_id()           --  gets the id of the DiskLibrary instance from commcell
 
+    move_mountpath()            --  To perform move mountpath operation
+
+    validate_mountpath()        --  To perform storage validation on mountpath
+
+    add_cloud_mount_path()      --  Adds a mount path to the cloud library
+
     add_mount_path()            --  adds the mount path on the local/ remote machine
 
     _get_library_properties()   --  gets the disk library properties
 
+    refresh()                   --  Refresh the properties of this disk library.
+
 DiskLibrary instance Attributes
 
     **media_agents_associated**  --  returns the media agents associated with the disk library
+    **library_properties**       --  Returns the dictionary consisting of the full properties of the library
 
 
 """
 from __future__ import absolute_import
 from __future__ import unicode_literals
-import uuid
+import uuid, time
 
 from base64 import b64encode
 
@@ -302,6 +323,63 @@ class MediaAgents(object):
             raise SDKException(
                 'Storage', '102', 'No media agent exists with name: {0}'.format(media_agent_name)
             )
+    
+    def delete(self, media_agent, force=False):
+        """Deletes the media agent from the commcell.
+
+            Args:
+                media_agent (str)  --  name of the Mediaagent to remove from the commcell
+                
+                force       (bool)     --  True if you want to delete media agent forcefully.
+
+            Raises:
+                SDKException:
+                    if type of the media agent name argument is not string
+
+                    if failed to delete Media agent
+
+                    if response is empty
+
+                    if response is not success
+
+                    if no media agent exists with the given name
+
+        """
+        if not isinstance(media_agent, basestring):
+            raise SDKException('Storage', '101')
+        else:
+            media_agent = media_agent.lower()
+
+            if self.has_media_agent(media_agent):
+                mediagent_id = self.all_media_agents[media_agent]['id']
+                mediagent_delete_service = self._commcell_object._services['MEDIA_AGENT'] % (mediagent_id)
+                if force:
+                    mediagent_delete_service += "?forceDelete=1"
+
+                flag, response = self._commcell_object._cvpysdk_object.make_request('DELETE', mediagent_delete_service)
+
+                error_code = 0
+                if flag:
+                    if 'errorCode' in response.json():
+                        o_str = 'Failed to delete mediaagent' 
+                        error_code = response.json()['errorCode'] 
+                        if error_code == 0:
+                            # initialize the mediaagents again
+                            # so the mediaagents object has all the mediaagents
+                            self.refresh()
+                        else:                                
+                            error_message = response.json()['errorMessage']
+                            if error_message:
+                                o_str += '\nError: "{0}"'.format(error_message)                        
+                            raise SDKException('Storage', '102', o_str)
+                    else:
+                        raise SDKException('Response', '102')
+                else:
+                    raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))
+            else:
+                raise SDKException(
+                    'Storage', '102', 'No Mediaagent exists with name: {0}'.format(media_agent)
+                )
 
     def refresh(self):
         """Refresh the media agents associated with the Commcell."""
@@ -336,6 +414,11 @@ class MediaAgent(object):
         self._MEDIA_AGENT = self._commcell_object._services['MEDIA_AGENT'] % (
             self._media_agent_name
         )
+
+        self._CLOUD_MEDIA_AGENT = self._commcell_object._services['CLOUD_MEDIA_AGENT'] % (
+            self._media_agent_id
+        )
+
         self._CREATE_TASK = self._commcell_object._services['CREATE_TASK']
         self._MEDIA_AGENTS = self._commcell_object._services['GET_MEDIA_AGENTS'] + "/{0}".format(
             self.media_agent_id
@@ -392,6 +475,11 @@ class MediaAgent(object):
         self._platform = None
         self._index_cache_enabled = None
         self._index_cache = None
+        self._is_power_mgmt_allowed = None
+        self._is_power_mgmt_supported = None
+        self._is_power_management_enabled = None
+        self._power_management_controller_name = None
+        self._power_status = None
 
         properties = self._get_media_agent_properties()
 
@@ -423,6 +511,155 @@ class MediaAgent(object):
             self._index_cache = mediaagent_list['mediaAgentProps']['mediaAgentIdxCacheProps'
                                                                    ]['cachePath']['path']
 
+        if mediaagent_list['powerManagementInfo']['isPowerMgmtSupported']:
+            self._is_power_mgmt_supported = mediaagent_list['powerManagementInfo']['isPowerMgmtSupported']
+
+        if self._is_power_mgmt_supported:
+
+            if mediaagent_list['powerManagementInfo']['isPowerManagementEnabled']:
+                self._is_power_management_enabled = mediaagent_list['powerManagementInfo']['isPowerManagementEnabled']
+
+            if mediaagent_list['powerManagementInfo']['isPowerMgmtAllowed']:
+                self._is_power_mgmt_allowed = mediaagent_list['powerManagementInfo']['isPowerMgmtAllowed']
+
+            if mediaagent_list['powerManagementInfo']['powerStatus']:
+                self._power_status = mediaagent_list['powerManagementInfo']['powerStatus']
+
+            if mediaagent_list['powerManagementInfo']['selectedCloudController']['clientName']:
+                self._power_management_controller_name = mediaagent_list['powerManagementInfo']['selectedCloudController']['clientName']
+
+    def enable_power_management(self, pseudo_client_name):
+        """
+            Enables power management using the provided cloud controller (pseudo client)
+
+                Args :
+                        pseudo_client_name : VSA pseudo client to be used as cloud controller
+                Raises:
+                        SDKException:
+                                    If response is not success
+                                    
+                                    If Power management is not supported
+        """
+        if self._is_power_mgmt_allowed:
+            client_obj = self._commcell_object._clients.get(pseudo_client_name)
+            pseudo_client_name_client_id = client_obj._get_client_id()
+
+            """
+            payLoad = '<EVGui_SetCloudVMManagementInfoReq hostId="' + self.media_agent_id + '" useMediaAgent="1"> <powerManagementInfo isPowerManagementEnabled="1" > <selectedCloudController clientId="' + PseudoClientName_client_id + '" clientName="' + \
+                      PseudoClientName + '"/></powerManagementInfo></EVGui_SetCloudVMManagementInfoReq>'
+            """
+            payLoad = '<EVGui_SetCloudVMManagementInfoReq hostId="{0}" useMediaAgent="1"> <powerManagementInfo isPowerManagementEnabled="1" > <selectedCloudController clientId="{1}" clientName="{2}"/></powerManagementInfo></EVGui_SetCloudVMManagementInfoReq>'.format(
+                self.media_agent_id, pseudo_client_name_client_id, pseudo_client_name)
+
+            response = self._commcell_object._qoperation_execute(payLoad)
+
+            if response['errorCode'] != 0:
+                raise SDKException('Response', '102', str(response))
+        else:
+            raise SDKException('Storage', '102', "Power management is not supported")
+
+    def _perform_power_operation(self, operation):
+        """
+            Performs power operation
+
+                Args :
+                        self : Object
+                        operation : Operation to perform
+                        
+                Raises:
+                        SDKException:
+                                        If operation is not 1 or 0
+                                            
+                                        If ower management is NOT enabled or NOT supported on MediaAgent
+
+                                        If API response is empty
+
+                                        If API response is not success
+        """
+        if not operation in ("1", "0"):
+            raise SDKException('Response', '102',
+                               "Invalid power operation type")
+
+        if self._is_power_management_enabled:
+            flag, response = self._commcell_object._cvpysdk_object.make_request(
+                'GET', self._CLOUD_MEDIA_AGENT + "/" + operation
+            )
+            if not flag:
+                raise SDKException('Response', '102',
+                                   str(response))
+            if response.json()['errorCode'] != 0:
+                raise SDKException('Response', '102', str(response))
+        else:
+            raise SDKException('Storage', '102',
+                               'Power management is NOT enabled or NOT supported')
+
+    def power_on(self, wait_till_online=True):
+        """
+            Power-on the MediaAgent
+
+                Args :
+                        self : Object
+                        wait_till_online :
+                                            True : Waits until the MediaAgent is online
+                                            False : Just submits the power-on request
+        """
+
+        if self.current_power_status not in ["Starting", "Started", "Online"]:
+            self._perform_power_operation("1")
+
+        if wait_till_online == True and self.current_power_status != "Online":
+            self.wait_for_power_status("Online")
+
+    def power_off(self, wait_till_stopped=True):
+        """
+            Power-off MediaAgent
+
+                Args :
+                        self : Object
+                        wait_till_stopped :
+                                            True : Waits until the MediaAgent is stopped
+                                            False : Just submits the power-off request
+        """
+
+        if self.current_power_status not in ["Stopping", "Stopped"]:
+            self._perform_power_operation("0")
+
+        if wait_till_stopped == True and self.current_power_status != "Stopped":
+            self.wait_for_power_status("Stopped")
+
+    def wait_for_power_status(self, expected_power_status, time_out_sec=600):
+        """
+            Waits until the expected power status not achieved
+
+                Args :
+                                        self : Object
+                                        expected_power_status : The expected power status as following.
+                                                                    Starting
+                                                                    Started
+                                                                    Online
+                                                                    Stopping
+                                                                    Stopped
+                                        time_out_sec : Maximum time to wait for the expected power status
+
+                                        Raises:
+                                                SDKException:
+                                                                If time_out_sec is not an integer and time_out_sec not None
+
+                                                                If expected power status is not achieved within time_out_sec time
+        """
+        if time_out_sec != None:
+            if not isinstance(time_out_sec, int):
+                raise SDKException('Storage', '102',
+                                   'Expected an integer value for [time_out_sec]')
+
+        start_time = time.time()
+        while self.current_power_status != expected_power_status:
+            time.sleep(10)
+            if time_out_sec != None:
+                if time.time() - start_time > time_out_sec:
+                    raise SDKException('Storage', '102',
+                                       'The expected power status is not achieved within expected time')
+
     def change_index_cache(self, old_index_cache_path, new_index_cache_path):
         """
         Begins a catalog migration job via the CreateTask end point.
@@ -555,6 +792,26 @@ class MediaAgent(object):
         """Treats the cache enabled value as a read-only attribute"""
         return self._index_cache_enabled
 
+    @property
+    def current_power_status(self):
+        """
+                Returns the power state of the MA.
+
+                    Args :
+                            self : Object
+                    Returns :
+                            str - Current power status of the MediaAgent as following
+                                    Starting : Power-on process in going on
+                                    Started : MA is powered-on successfully but still not synced with CS
+                                    Online : Powered-on and synced with CS. MA is ready to use.
+                                    Stopping : Power-off operation is going on.
+                                    Stopped : MA is powered-off
+                                    Unknown : MA power status is still not synced with cloud provider. MA discovery is going on or power state sync with happening with cloud provider or something is NOT right.
+        """
+        self.refresh()
+        power_status = {0: 'Unknown', 1: 'Starting', 2: 'Started', 3: 'Online', 4: 'Stopping', 5: 'Stopped'}
+        return power_status.get(self._power_status)
+
     def refresh(self):
         """Refresh the properties of the MediaAgent."""
         self._initialize_media_agent_properties()
@@ -895,6 +1152,230 @@ class DiskLibrary(object):
             self.library_name, self._commcell_object.commserv_name
         )
 
+    def move_mountpath(self, mountpath_id, source_device_path,
+                       source_mediaagent_id, target_device_path, target_mediaagent_id):
+
+        """ To perform move mountpath operation
+        Args:
+            mountpath_id  (int)   --  Mountpath Id that need to be moved.
+
+            source_device_path (str)   -- Present Mountpath location
+
+            source_mediaagent_id    (int)   -- MediaAgent Id on which present mountpath exists
+
+            target_device_path    (str)   -- New Mountpath location
+
+            target_mediaagent_id    (int)   -- MediaAgent Id on which new mountpath exists
+
+        Returns:
+            instance of the Job class for this move mountpath job
+
+        Raises
+            Exception:
+                - if argument datatype is invalid
+
+                - if API response error code is not 0
+
+                - if response is empty
+
+                - if response code is not as expected
+        """
+
+        if not (isinstance(mountpath_id, int) and
+                isinstance(source_mediaagent_id, int) and
+                isinstance(target_mediaagent_id, int) and
+                isinstance(target_device_path, basestring) and
+                isinstance(source_device_path, basestring)):
+            raise SDKException('Storage', '101')
+
+        request_xml = """<TMMsg_CreateTaskReq>
+                        <taskInfo>
+                            <task initiatedFrom="1" ownerId="1" sequenceNumber="0" taskId="0" taskType="1">
+                                <taskFlags disabled="0" />
+                            </task>
+                            <associations mountPathId="{1}" />
+                            <subTasks subTaskOperation="1">
+                                <subTask operationType="5017" subTaskType="1" />
+                                <options> <adminOpts> <libraryOption>
+                                    <library libraryId="{0}" />
+                                    <moveMPOption>
+                                        <mountPathMoveList mountPathId="{1}" sourceDevicePath="{2}" 
+                                        sourcemediaAgentId="{3}" targetDevicePath="{4}" targetMediaAgentId="{5}">
+                                        <credential credentialId="0" credentialName=" " />
+                                        </mountPathMoveList>
+                                    </moveMPOption>
+                                </libraryOption> </adminOpts> </options>
+                            </subTasks>
+                        </taskInfo>
+                    </TMMsg_CreateTaskReq>""".format(self.library_id, mountpath_id, source_device_path,
+                                                     source_mediaagent_id, target_device_path, target_mediaagent_id)
+
+        flag, response = self._commcell_object._cvpysdk_object.make_request(
+            'POST', self._commcell_object._services['CREATE_TASK'], request_xml)
+
+        if flag:
+            if response.json():
+                if "jobIds" in response.json():
+                    from cvpysdk.job import Job
+                    return Job(self._commcell_object, response.json()['jobIds'][0])
+
+                if "errorCode" in response.json():
+                    error_message = response.json()['errorMessage']
+                    o_str = 'Error: "{0}"'.format(error_message)
+                    raise SDKException('Commcell', '105', o_str)
+
+                else:
+                    raise SDKException('Commcell', '105')
+
+            else:
+                raise SDKException('Response', '102')
+        else:
+            raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))
+
+    def validate_mountpath(self, mountpath_drive_id, media_agent):
+
+        """ To perform storage validation on mountpath
+        Args:
+            mountpath_drive_id  (int)   --  Drive Id of mountpath that need to be validate.
+
+            media_agent (str)   -- MediaAgent on which Mountpath exists
+
+        Returns:
+            instance of the Job class for this storage validation job
+
+        Raises
+            Exception:
+                - if argument datatype is invalid
+
+                - if API response error code is not 0
+
+                - if response is empty
+
+                - if response code is not as expected
+        """
+
+        if not (isinstance(mountpath_drive_id, int) and
+                isinstance(media_agent, basestring)):
+            raise SDKException('Storage', '101')
+
+
+        request_xml = """<TMMsg_CreateTaskReq>
+                        <taskInfo taskOperation="1">
+                            <task associatedObjects="0" description="Storage Validation - Automation" initiatedFrom="1" 
+                            isEZOperation="0" isEditing="0" isFromCommNetBrowserRootNode="0" ownerId="1" ownerName="" 
+                            policyType="0" runUserId="1" sequenceNumber="0" taskType="1">
+                            <taskFlags notRunnable="0" />
+                            </task>
+                            <subTasks subTaskOperation="1">
+                                <subTask flags="0" operationType="4013" subTaskId="1" subTaskOrder="0" subTaskType="1"/>
+                                <options originalJobId="0">
+                                    <adminOpts>
+                                        <libraryOption  operation="13" validationFlags="0" validattionReservedFlags="0">
+                                            <library libraryId="{0}" />
+                                            <mediaAgent mediaAgentName="{1}" />
+                                            <driveIds driveId="{2}" />
+                                            <validateDrive chunkSize="16384" chunksTillEnd="0" fileMarkerToStart="2"
+                                             numberOfChunks="2" threadCount="2" volumeBlockSize="64" />
+                                        </libraryOption> </adminOpts> </options> </subTasks>  </taskInfo>
+                            </TMMsg_CreateTaskReq>""".format(self.library_id, media_agent, mountpath_drive_id)
+
+        flag, response = self._commcell_object._cvpysdk_object.make_request(
+            'POST', self._commcell_object._services['CREATE_TASK'], request_xml
+        )
+
+        if flag:
+            if response.json():
+                if "jobIds" in response.json():
+                    from cvpysdk.job import Job
+                    return Job(self._commcell_object, response.json()['jobIds'][0])
+
+                if "errorCode" in response.json():
+                    error_message = response.json()['errorMessage']
+                    o_str = 'Error: "{0}"'.format(error_message)
+                    raise SDKException('Commcell', '105', o_str)
+
+                else:
+                    raise SDKException('Commcell', '105')
+
+            else:
+                raise SDKException('Response', '102')
+        else:
+            raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))
+
+    def add_cloud_mount_path(self, mount_path, media_agent, username, password, server_type):
+        """ Adds a mount path to the cloud library
+
+        Args:
+            mount_path  (str)   -- cloud container or bucket.
+
+            media_agent (str)   -- MediaAgent on which mountpath exists
+
+            username    (str)   -- Username to access the mount path in the format <Service Host>//<Account Name>
+            Eg: s3.us-west-1.amazonaws.com//MyAccessKeyID. For more information refer http://documentation.commvault.com/commvault/v11/article?p=97863.htm.
+
+            password    (str)   -- Password to access the mount path
+
+            server_type  (int)   -- provide cloud library server type
+                                    Eg: 3-Microsoft Azure Storage . For more information refer http://documentation.commvault.com/commvault/v11/article?p=97863.htm.
+        Returns:
+            None
+
+        Raises
+            Exception:
+                - if mountpath or mediaagent or username or password or servertype arguments dataype is invalid
+
+                - if servertype input data is incorrect
+
+                - if API response error code is not 0
+
+                - if response is empty
+
+                - if response code is not as expected
+            """
+
+        if not (isinstance(mount_path, basestring) or isinstance(media_agent, basestring)
+                or isinstance(username, basestring) or isinstance(password, basestring)
+                or isinstance(server_type, int)):
+            raise SDKException('Storage', '101')
+
+        request_json = {
+            "isConfigRequired": 1,
+            "library": {
+                "opType": 4,
+                "isCloud": 1,
+                "mediaAgentName": media_agent,
+                "libraryName": self._library_name,
+                "mountPath": mount_path,
+                "loginName": username,
+                "password": b64encode(password.encode()).decode(),
+                "serverType": server_type
+            }
+        }
+
+        exec_command = self._commcell_object._services['LIBRARY']
+        flag, response = self._commcell_object._cvpysdk_object.make_request(
+            'POST', exec_command, request_json
+        )
+
+        if flag:
+            if response.json():
+                if 'library' in response.json():
+                    _response = response.json()['library']
+
+                    if 'errorCode' in _response:
+                        if _response['errorCode'] != 0:
+                            raise SDKException('Storage', '102', _response['errorMessage'])
+                else:
+                    raise SDKException('Response', '102')
+            else:
+                raise SDKException('Response', '102')
+        else:
+            _stdout = 'Failed to add mount path [{0}] for library [{1}] with error: \n [{2}]'
+            _stderr = self._commcell_object._update_response_(response.text)
+            raise SDKException('Response', '101', _stdout.format(mount_path,
+                                                                 self._library_name,
+                                                                 _stderr))
+
     def _get_library_properties(self):
         """Gets the disk library properties.
 
@@ -931,6 +1412,10 @@ class DiskLibrary(object):
         libraries = DiskLibraries(self._commcell_object)
         return libraries.get(self.library_name).library_id
 
+    def refresh(self):
+        """Refresh the properties of this disk library."""
+        self._library_properties = self._get_library_properties()
+
     def add_mount_path(self, mount_path, media_agent, username='', password=''):
         """ Adds a mount path [local/remote] to the disk library
 
@@ -1024,6 +1509,12 @@ class DiskLibrary(object):
         """Treats the library id as a read-only attribute."""
         return self._library_id
 
+    @property
+    def library_properties(self):
+        """Returns the dictionary consisting of the full properties of the library"""
+        self.refresh()
+        return self._library_properties
+
     @property
     def mount_path(self):
         """Treats the library id as a read-only attribute."""
@@ -1033,32 +1524,98 @@ class DiskLibrary(object):
     def media_agent(self):
         """Treats the library id as a read-only attribute."""
         return self.mediaagent
+        
+    def share_mount_path(self, new_media_agent, new_mount_path, **kwargs):
+        """
+        Method to share a mountpath to a disklibrary
 
-    def shareMountpath(self, newMA, newmountpath):
-        """
-        method to specify share mountpath to a disklibrary
+        Args:
+        
+            new_media_agent (str)   -- Media agent which is accessing the shared mount path
+            
+            new_mount_path  (int)   -- Mount path to be shared
+            
+            \*\*kwargs  (dict)  --  Optional arguments
+
+                    Available kwargs Options:
+            
+                        media_agent     (str)   -- Media agent associated with library
+                        
+                        library_name    (str)   -- Name of the library which has the mount path
+                    
+                        mount_path      (str)   -- Mount path to be shared
+                        
+                        access_type     (int)   -- The access type of the shared mount path
+
+                                                    Read Device Access = 4
+                                                    
+                                                    Read/ Write Device Access = 6
+                                                    
+                                                    Read Device Access with Preferred = 12
+                                                    
+                                                    Read/Write Device Access with Preferred = 14
+                                                    
+                                                    Data Server - IP Read = 20
+                                                    
+                                                    Data Server - IP Read/ Write = 22
+                                                    
+                                                    Data Server - FC Read = 36
+                                                    
+                                                    Data Server - FC Read/ Write = 38
+                                                    
+                                                    Data Server - iSCSI Read = 132
+                                                    
+                                                    Data Server - iSCSI Read/ Write = 134
+                                                    
+                                                    Note: For the Data Server device access type,
+                                                          enter the local path provided in the library/mountPath
+                                                          parameter in the libNewProp/mountPath parameter also.
+                        
+
+                        username        (str)   -- Username to access the mount path, if UNC
+
+                        password        (str)   -- Password to access the mount path, if UNC
+
+        Returns:
+            None
+
+        Raises
+            Exception:
+                - if any of the parameter's dataype is invalid
+
+                - if API response error code is not 0
+
+                - if response is empty
+
+                - if response code is not as expected
         """
+        
+        media_agent = kwargs.get('media_agent', self.mediaagent)
+        library_name = kwargs.get('library_name', self.library_name)
+        mount_path = kwargs.get('mount_path', self.mountpath)
+        access_type = kwargs.get('access_type', 22)
+        username = kwargs.get('username', '')
+        password = kwargs.get('password', '')
+       
         self._EXECUTE = self._commcell_object._services['EXECUTE_QCOMMAND']
         self.library = {
             "opType": 64,
-            "mediaAgentName": "%s" %
-                              self.mediaagent,
-            "libraryName": "%s" %
-                           self.library_name,
+            "mediaAgentName": media_agent,
+            "libraryName": library_name,
             "mountPath": "%s" %
-                         self.mountpath}
-        self.libNewprop = {
-            "deviceAccessType": 22,
-            "password": "",
-            "loginName": "",
-            "mediaAgentName": newMA,
-            "mountPath": "{}".format(newmountpath),
+                         mount_path}
+        self.lib_new_prop = {
+            "deviceAccessType": access_type,
+            "password": password,
+            "loginName": username,
+            "mediaAgentName": new_media_agent,
+            "mountPath": "{}".format(new_mount_path),
             "proxyPassword": ""}
         request_json = {
             "EVGui_ConfigureStorageLibraryReq":
                 {
                     "library": self.library,
-                    "libNewProp": self.libNewprop
+                    "libNewProp": self.lib_new_prop
                 }
         }
 
